<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\Category;
use SGalinski\SgNews\Domain\Model\News;
use SGalinski\SgNews\Domain\Repository\CategoryRepository;
use SGalinski\SgNews\Domain\Repository\NewsRepository;
use SGalinski\SgNews\Domain\Repository\TagRepository;
use SGalinski\SgNews\Service\HeaderMetaDataService;
use TYPO3\CMS\Core\Charset\CharsetConverter;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;

/**
 * Controller that handles the news single view page
 */
class SingleViewController extends AbstractController {
	/**
	 * @var CategoryRepository
	 */
	protected $categoryRepository;

	/**
	 * @var TagRepository
	 */
	protected $tagRepository;

	/**
	 * @var NewsRepository
	 */
	protected $newsRepository;

	/**
	 * @var CharsetConverter
	 */
	protected $characterSetConverter;

	/**
	 * Renders the news single view
	 *
	 * @return void
	 * @throws \InvalidArgumentException
	 */
	public function singleViewAction() {
		/** @var News $news */
		$currentId = (int) $GLOBALS['TSFE']->id;
		$news = $this->newsRepository->findByUid($currentId);
		if (!$news) {
			return;
		}

		/** @var Category $newsCategory */
		$newsCategory = $this->categoryRepository->findByUid($news->getPid());
		if (!$newsCategory) {
			return;
		}

		$newsMetaData = $this->getMetaDataForNews($news, $newsCategory);
		if (!version_compare(ExtensionManagementUtility::getExtensionVersion('sg_seo'), '5.0.0', '>=')) {
			if ($newsMetaData['image']) {
				HeaderMetaDataService::addOgImageToHeader($newsMetaData['image']);
			} elseif ($newsMetaData['teaserImage']) {
				HeaderMetaDataService::addOgImageToHeader($newsMetaData['teaserImage']);
			}
		}

		$previousNews = $this->newsRepository->findPreviousNewsEntryFromCurrentNews($news, $this->settings['sortBy'])
			->getFirst();
		$nextNews = $this->newsRepository->findNextNewsEntryFromCurrentNews($news, $this->settings['sortBy'])
			->getFirst();

		$this->view->assignMultiple(
			[
				'newsMetaData' => $newsMetaData,
//				'similarNewsMetaData' => $similarNewsMetaData,
				'previousNews' => $previousNews,
				'nextNews' => $nextNews,
				'newsAuthor' => $news->getNewsAuthor(),
			]
		);
	}

	/**
	 * @param CategoryRepository $categoryRepository
	 */
	public function injectCategoryRepository(CategoryRepository $categoryRepository
	) {
		$this->categoryRepository = $categoryRepository;
	}

	/**
	 * @param CharsetConverter $characterSetConverter
	 */
	public function injectCharacterSetConverter(CharsetConverter $characterSetConverter) {
		$this->characterSetConverter = $characterSetConverter;
	}

	/**
	 * @param NewsRepository $newsRepository
	 */
	public function injectNewsRepository(NewsRepository $newsRepository) {
		$this->newsRepository = $newsRepository;
	}

	/**
	 * @param TagRepository $tagRepository
	 */
	public function injectTagRepository(TagRepository $tagRepository) {
		$this->tagRepository = $tagRepository;
	}
}
